@@ -60,17 +60,25 @@
 NAMESPACE_SIPWITCH
 using namespace UCOMMON_NAMESPACE;
 
-class __EXPORT service : public mempager
+class __EXPORT service : public memalloc
 {
 public:
 	typedef treemap<char *>keynode;
-	
+
 	typedef struct {
 		const char *key;
 		const char *value;
 	} define;
 
-	class pointer 
+	class __EXPORT usernode
+	{
+	public:
+		service::keynode *keys;
+		service *heap;
+		usernode();
+	};
+
+	class __EXPORT pointer 
 	{
 	private:
 		keynode *node;
@@ -125,10 +133,10 @@ public:
         virtual ~callback();
 
 		inline static void *alloc(service *cfgp, size_t size)
-			{return cfgp->alloc_locked(size);};
+			{return cfgp->alloc(size);};
 
 		inline static char *dup(service *cfgp, const char *s)
-			{return cfgp->dup_locked(s);};
+			{return cfgp->dup(s);};
 
 		inline static bool isConfigured(void)
 			{return service::cfg != NULL;};
@@ -214,6 +222,7 @@ protected:
 	stringbuf<1024> buffer;
 
 	void setHeader(const char *header);
+	void clearId(void);
 
 private:
 	void __LOCAL addAttributes(keynode *node, char *astr);
