@@ -147,7 +147,6 @@ void messages::update(const char *uid)
 bool messages::publish(const char *to, const char *reply, const char *from, caddr_t text, size_t len, const char *msgtype)
 {
 	message *msg;
-	char *ep;
 
 	if(!msgtype)
 		msgtype = "text/plain";
@@ -215,7 +214,6 @@ bool messages::deliver(message *msg)
 
 	linked_pointer<registry::target> tp;
 	registry::mapped *rr = registry::access(msg->user);
-	unsigned path = NamedObject::keyindex(msg->user, keysize);
 	osip_message_t *im;
 	time_t now;
 	unsigned msgcount = 0;
