@@ -134,18 +134,22 @@ extern "C" int main(int argc, char **argv)
 
 	// for deaemon env usually loaded from /etc/defaults or /etc/sysconfig
 
+	cp = getenv("GROUP");
+	if(cp && *cp)
+		user = strdup(cp);
+
 	cp = getenv("PRIORITY");
-	if(cp)
+	if(cp && *cp)
 		priority = atoi(cp);
 
 	cp = getenv("VERBOSE");
-	if(cp) {
+	if(cp && *cp) {
 		warned = true;
 		verbose = atoi(cp);
 	}
 
 	cp = getenv("CFGFILE");
-	if(cp)
+	if(cp && *cp)
 		cfgfile = strdup(cp);
 
 	while(NULL != *(++argv)) {
@@ -194,7 +198,7 @@ extern "C" int main(int argc, char **argv)
 		if(!stricmp(*argv, "-version"))
 			server::version();
 
-		if(!strcmp(*argv, "-u") || !stricmp(*argv, "-user")) {
+		if(!strcmp(*argv, "-u") || !stricmp(*argv, "-user") || !stricmp(*argv, "-group") || !stricmp(*argv, "-g")) {
 			user = *(++argv);
 			if(!user) {
 				fprintf(stderr, "*** sipw: user option missing\n");
@@ -203,6 +207,11 @@ extern "C" int main(int argc, char **argv)
 			continue;
 		}
 
+		if(!strnicmp(*argv, "-group=", 7)) {
+			user = *argv + 7;
+			continue;
+		} 
+
 		if(!strnicmp(*argv, "-user=", 6)) {
 			user = *argv + 6;
 			continue;
