@@ -596,26 +596,30 @@ extern "C" int main(int argc, char **argv)
 
 	// for deaemon env usually loaded from /etc/defaults or /etc/sysconfig
 
+	cp = getenv("GROUP");
+	if(cp && *cp)
+		user = strdup(cp);
+
 	cp = getenv("CONCURRENCY");
-	if(cp)
+	if(cp && *cp)
 		concurrency = atoi(cp);
 
 	cp = getenv("PRIORITY");
-	if(cp)
+	if(cp && *cp)
 		priority = atoi(cp);
 
 	cp = getenv("VERBOSE");
-	if(cp) {
+	if(cp && *cp) {
 		warned = true;
 		verbose = atoi(cp);
 	}
 
 	cp = getenv("CFGFILE");
-	if(cp)
+	if(cp && *cp)
 		cfgfile = strdup(cp);
 
 	cp = getenv("PLUGINS");
-	if(cp)
+	if(cp && *cp)
 		plugins = strdup(cp);
 	else
 		plugins = (char *)"none";
@@ -704,7 +708,7 @@ extern "C" int main(int argc, char **argv)
 		if(!stricmp(*argv, "-version")) 
 			server::version();
 
-		if(!strcmp(*argv, "-u") || !stricmp(*argv, "-user")) {
+		if(!strcmp(*argv, "-u") || !stricmp(*argv, "-user") || !strcmp(*argv, "-g") || !stricmp(*argv, "-group")) {
 			user = *(++argv);
 			if(!user) {
 				fprintf(stderr, "*** sipw: user option missing\n");
@@ -718,6 +722,11 @@ extern "C" int main(int argc, char **argv)
 			continue;
 		} 
 
+		if(!strnicmp(*argv, "-group=", 7)) {
+			user = *argv + 7;
+			continue;
+		} 
+
 		if(!stricmp(*argv, "-help") || !stricmp(*argv, "-?")) 
 			server::usage();
 
