@@ -165,7 +165,7 @@ bool server::create(const char *id, keynode *node)
 	assert(id != NULL && *id != 0);
 	assert(node != NULL);
 
-	keymap *map = (keymap *)alloc_locked(sizeof(keymap));
+	keymap *map = (keymap *)alloc(sizeof(keymap));
 	unsigned path = NamedObject::keyindex(id, CONFIG_KEY_SIZE);
 	
 	if(find(id))
@@ -215,13 +215,13 @@ bool server::confirm(const char *user)
 		memset(extmap, 0, sizeof(keynode *) * range);
 	}
 	profiles = NULL;
-	mp = (caddr_t)alloc_locked(sizeof(profile));
+	mp = (caddr_t)alloc(sizeof(profile));
 	ppd = new(mp) profile(&profiles);
 	String::set(ppd->value.id, sizeof(ppd->value.id), "*");
 	ppd->value.level = 1;
 	ppd->value.features = USER_PROFILE_DEFAULT;
 	
-	mp = (caddr_t)alloc_locked(sizeof(profile));
+	mp = (caddr_t)alloc(sizeof(profile));
 	pp = new(mp) profile(&profiles);
 	memcpy(&pp->value, &ppd->value, sizeof(profile_t));
 	String::set(pp->value.id, sizeof(pp->value.id), "restricted");
@@ -278,17 +278,17 @@ bool server::confirm(const char *user)
 
 	fsys::close(dir);
 
-	mp = (caddr_t)alloc_locked(sizeof(cidr));
+	mp = (caddr_t)alloc(sizeof(cidr));
 	new(mp) cidr(&acl, "127.0.0.0/8", "loopback");
 
-	mp = (caddr_t)alloc_locked(sizeof(cidr));
+	mp = (caddr_t)alloc(sizeof(cidr));
 	new(mp) cidr(&acl, "::1", "loopback");
 
 	node = access->getFirst();
 	while(node) {
 		id = node->getId();
 		if(id && node->getPointer()) {
-			mp = (caddr_t)alloc_locked(sizeof(cidr));
+			mp = (caddr_t)alloc(sizeof(cidr));
 			new(mp) cidr(&acl, node->getPointer(), id);
 		}
 		node.next();
@@ -306,7 +306,7 @@ bool server::confirm(const char *user)
 			id = NULL;
 
 		if(leaf && id && !strcmp(node->getId(), "profile")) {
-			mp = (caddr_t)alloc_locked(sizeof(profile));
+			mp = (caddr_t)alloc(sizeof(profile));
 			pp = new(mp) profile(&profiles);
 			memcpy(&pp->value, &ppd->value, sizeof(profile_t));
 			String::set(pp->value.id, sizeof(pp->value.id), id);
@@ -338,9 +338,9 @@ bool server::confirm(const char *user)
 				else
 					digest::md5(digest);
 				if(digest[0]) {
-					mp = (caddr_t)alloc_locked(sizeof(keynode));
+					mp = (caddr_t)alloc(sizeof(keynode));
 					leaf = new(mp) keynode(node, (char *)"digest");
-					leaf->setPointer(dup_locked(*digest));
+					leaf->setPointer(dup(*digest));
 				}
 			}
 			leaf = node->leaf("extension");
@@ -360,6 +360,17 @@ void server::release(keynode *node)
 		locking.release();
 }
 
+void server::release(usernode& user)
+{
+	if(user.heap)
+		delete user.heap;
+	else
+		service::release(user.keys);
+
+	user.keys = NULL;
+	user.heap = NULL;
+}
+
 void server::release(cidr *access)
 {
 	if(access)
@@ -438,7 +449,7 @@ service::keynode *server::getConfig(void)
 	return (keynode *)cfg;
 }
 
-service::keynode *server::getExtension(const char *id)
+void server::getExtension(const char *id, usernode& user)
 {
 	assert(id != NULL && *id != 0);
 	assert(cfg != NULL);
@@ -450,11 +461,13 @@ service::keynode *server::getExtension(const char *id)
 	keynode *node = NULL;
 	const char *cp;
 
+	server::release(user);
+
 	locking.access();
 	cfgp = static_cast<server *>(cfg);
 	if(!cfgp) {
 		locking.release();
-		return NULL;
+		return;
 	}
 
 	if(range && ext >= prefix && ext < prefix + range)
@@ -469,7 +482,7 @@ service::keynode *server::getExtension(const char *id)
 	}
 	if(!node)
 		locking.release();
-	return node;
+	user.keys = node;
 }
 
 Socket::address *server::getContact(const char *uid)
@@ -477,16 +490,18 @@ Socket::address *server::getContact(const char *uid)
 	assert(uid != NULL && *uid != 0);
 	assert(cfg != NULL);
 
-	keynode *node = getProvision(uid);
+	usernode user;
+
+	getProvision(uid, user);
 	Socket::address *addr = NULL;
 
-	if(!node)
+	if(!user.keys)
 		return NULL;
 
-	node = node->leaf("contact");
+	service::keynode *node = user.keys->leaf("contact");
 	if(node)
 		addr = stack::getAddress(node->getPointer());
-	locking.release();
+	server::release(user);
 	return addr;
 }
 
@@ -530,16 +545,15 @@ unsigned server::getForwarding(const char *uid)
 {
 	assert(uid != NULL);
 
-	keynode *node = getProvision(uid);
-	unsigned mask = forwarding(node);
+	usernode user;
+	getProvision(uid, user);
+	unsigned mask = forwarding(user.keys);
 	
-	if(node)
-		release(node);
-
+	server::release(user);
 	return mask;
 }
 
-service::keynode *server::getProvision(const char *uid)
+void server::getProvision(const char *uid, usernode& user)
 {
 	assert(uid != NULL && *uid != 0);
 	assert(cfg != NULL);
@@ -550,18 +564,20 @@ service::keynode *server::getProvision(const char *uid)
 	unsigned prefix = registry::getPrefix();
 	unsigned ext = atoi(uid);
 
+	server::release(user);
+
 	locking.access();
 	cfgp = static_cast<server*>(cfg);
 	if(!cfgp) {
 		locking.release();
-		return NULL;
+		return;
 	}
 	node = cfgp->find(uid);
 	if(!node && range && ext >= prefix && ext < prefix + range)
 		node = cfgp->extmap[ext - prefix];
 	if(!node)
 		locking.release();
-	return node;
+	user.keys = node;
 }
 
 void server::utils(const char *uid)
